require "import"
import "android.media.MediaPlayer"
import "java.io.File"
import "android.app.NotificationManager"
import "android.app.NotificationChannel"
import "android.app.PendingIntent"
import "android.content.Context"
import "android.content.Intent"
import "android.content.BroadcastReceiver"
import "android.content.IntentFilter"
import "android.app.Notification"
import "java.util.Random"
import "android.os.Vibrator"
import "android.speech.tts.TextToSpeech"

local folderPath = "/storage/emulated/0/Download"

function getAllSongs(folderPath)
    local folder = File(folderPath)
    local files = folder.listFiles()
    local songList = {}

    if files then
        for i = 0, #files - 1 do
            local file = files[i]
            if file.isFile() and file.getName():lower():match("%.mp3$") then
                table.insert(songList, file.getAbsolutePath())
            end
        end
    end

    return songList
end

songs = getAllSongs(folderPath)
if #songs == 0 then
    return false
end

local CHANNEL_ID = "music_player_channel"
local NOTIF_ID = 1
local STOP_ACTION = "STOP_MUSIC"
local context = activity or service or application

math.randomseed(os.time())

-- Initialize Google TTS
local tts
local ttsReady = false

function initializeTTS(callback)
    tts = TextToSpeech(context, TextToSpeech.OnInitListener {
        onInit = function(status)
            if status == TextToSpeech.SUCCESS then
                local googleTTS = "com.google.android.tts"
                if tts.setEngineByPackageName then
                    tts.setEngineByPackageName(googleTTS)
                end
                ttsReady = true
                if callback then callback() end
            end
        end
    })
end

function speak(text, callback)
    if ttsReady and tts then
        tts.speak(text, TextToSpeech.QUEUE_FLUSH, nil, nil)
        if callback then
            task(1000, callback)
        end
    elseif callback then
        callback()
    end
end

-- Function for Vibration
function vibrate()
    local vibrator = context.getSystemService(Context.VIBRATOR_SERVICE)
    if vibrator and vibrator.hasVibrator() then
        if android and android.os and android.os.VibrationEffect then
            local effect = android.os.VibrationEffect.createOneShot(500, android.os.VibrationEffect.DEFAULT_AMPLITUDE)
            vibrator.vibrate(effect)
        else
            vibrator.vibrate(500)
        end
    end
end

function createNotificationChannel()
    local manager = context.getSystemService(Context.NOTIFICATION_SERVICE)
    local channel = NotificationChannel(CHANNEL_ID, "Music Player", NotificationManager.IMPORTANCE_LOW)
    manager.createNotificationChannel(channel)
end

function showNotification()
    local manager = context.getSystemService(Context.NOTIFICATION_SERVICE)

    local stopIntent = Intent(STOP_ACTION)
    local pendingIntent = PendingIntent.getBroadcast(context, 0, stopIntent, PendingIntent.FLAG_IMMUTABLE)

    local builder = Notification.Builder(context, CHANNEL_ID)
        .setContentTitle("Music Player")
        .setContentText("Playing music...")
        .setSmallIcon(android.R.drawable.ic_media_play)
        .addAction(android.R.drawable.ic_media_pause, "STOP", pendingIntent)
        .setOngoing(true)

    manager.notify(NOTIF_ID, builder.build())
end

function removeNotification()
    local manager = context.getSystemService(Context.NOTIFICATION_SERVICE)
    manager.cancel(NOTIF_ID)
end

function stopMusic()
    if mediaPlayer then
        mediaPlayer.stop()
        mediaPlayer.release()
        mediaPlayer = nil
        removeNotification()
        speak("Music player deactivated", vibrate)
    end
end

local StopReceiver = luajava.bindClass("android.content.BroadcastReceiver")

local stopReceiverInstance = luajava.new(StopReceiver, {
    onReceive = function(context, intent)
        if intent.getAction() == STOP_ACTION then
            stopMusic()
        end
    end
})

local filter = IntentFilter(STOP_ACTION)
context.registerReceiver(stopReceiverInstance, filter)

local playedSongs = {}
local totalSongs = #songs
local isFirstPlay = true -- Flag para sa unang kanta lang may voice feedback

function playRandomSong()
    if mediaPlayer then
        mediaPlayer.stop()
        mediaPlayer.release()
    end

    if #playedSongs == totalSongs then
        playedSongs = {} -- Reset the played list when all songs are played
    end

    local availableSongs = {}
    for i, song in ipairs(songs) do
        if not playedSongs[song] then
            table.insert(availableSongs, song)
        end
    end

    if #availableSongs == 0 then
        return
    end

    local selectedSong = availableSongs[math.random(1, #availableSongs)]
    playedSongs[selectedSong] = true

    mediaPlayer = MediaPlayer()
    mediaPlayer.setDataSource(selectedSong)
    mediaPlayer.prepareAsync()

    mediaPlayer.setOnPreparedListener(function(mp)
        if isFirstPlay then
            -- ✅ Voice feedback sa pagsisimula lang ng player (unang pindot)
            speak("Music player activated", function()
                vibrate()
                mp.start()
                showNotification()
                isFirstPlay = false -- ❌ Para hindi na magsalita ulit sa susunod na kanta
            end)
        else
            -- ❌ Walang voice feedback kapag lilipat ng kanta
            mp.start()
            showNotification()
        end
    end)

    mediaPlayer.setOnCompletionListener(function(mp)
        playRandomSong() -- ✅ Wala nang voice feedback kapag lumilipat ng kanta
    end)
end

createNotificationChannel()

initializeTTS(function()
    playRandomSong()
end)

return true